NAME_PROP = "name"
USE_CASE_PROP = "useCase"
STATELESS_PROP = "stateless"
KEEP_IN_CACHE_PROP = "keepInCache"
LAST_FLOW_PROP = "lastUse"

CALLBACK_ERROR_STRATEGY_ARG = "on_key_changed_callback_error_strategy"
KEEP_IN_CACHE_ARG = "keep_in_cache"
STATELESS_ARG = "stateless"
USE_CASE_ARG = "use_case"
SECRET_KEY_TYPE_NAME = "SecretKey"
SECRET_KEY_PAIR_TYPE_NAME = "SecretKeyPair"
SECRET_KEY_DEFINITION_TYPE_NAME = "SecretKeyDefinition"
