ERROR_STRATEGY_TYPE_ERR_MSG = '"on_key_changed_callback_error_strategy" property is not type of "OnKeyChangedCallbackErrorStrategy"'
KEEP_IN_CACHE_NOT_BOOL_ERR_MSG = '"keep_in_cache" property is not boolean'
STATELESS_NOT_BOOL_ERR_MSG = '"stateless" property is not boolean'
USE_CASE_PROP_TYPE_ERR = '"use_case" property is not type of "SecretKeyUseCase"'
SHOULD_NOT_CONTAINS_ARGS_MSG = '"keys_store" signature should not contains args'
STORE_IS_NOT_CALLABLE_MSG = '"keys_store" is not callable'
NAME_PROPERTY_IS_EMPTY_MSG = '"name" property is empty'
